import os
from notebook.notebookapp import NotebookApp

NOTEBOOK_DIRECTORY = os.path.join(
    os.path.dirname(__file__))

def main():
    NotebookApp.launch_instance(notebook_dir=NOTEBOOK_DIRECTORY)

if __name__ == "__main__":
    main()


"""Create a report of csv file comparisons.
"""

import os

import numpy as np
import pandas as pd

from jinja2 import Template

from IPython.display import HTML, display, Markdown

from csv_compare import compare as _compare


def _create_comparison_table(data):
    """Create a table for displaying the results.
    """
    keys = np.sort(list(data.keys()))
    return pd.DataFrame(
        columns=['Result'],
        index=keys,
        data=[data[key] for key in keys])


def create_reports(input_directory, output_directory):
    """Create a report for every timestamp within the input directory.
    """
    all_comparison_data = _compare(input_directory)

    for timestamp, comparison_data in all_comparison_data.items():
        
        key = comparison_data.keys()
        assert (
            len(key) == 1
        ), "More than one comparison report not yet implemented"
        centre_names = list(key)[0]

        field_overviews = {
            centre_name: pd.read_csv(os.path.join(
                input_directory, timestamp, centre_name, 'fields.csv'
            ), index_col=0)
            for centre_name in centre_names
        }        

        output_filepath = r"{}\\{}_timestamp_{}.html".format(
            output_directory, os.path.basename(input_directory), timestamp)
        _create_report(
            output_filepath, comparison_data[centre_names], field_overviews)


def _create_report(output_filepath, comparison_data, field_overviews):
    """Create the report according to report_template.html.
    """
    nbccc_fields = field_overviews['nbccc']
    rccc_fields = field_overviews['rccc']
    tests = _create_comparison_table(comparison_data)

    with open('report_template.html') as file:
        template_string = file.read()

    template = Template(template_string)
    html = template.render(
        nbccc_fields=nbccc_fields.to_html(),
        rccc_fields=rccc_fields.to_html(),
        results=tests.to_html()
    )

    display(Markdown('# Filepath: {}'.format(output_filepath)))
    display(HTML(html))

    with open(output_filepath, "w") as html_file:
        html_file.write(html)
    


"""Pull patient field data from Mosaiq SQL and save to csv.
"""

import os
import json
import datetime

import numpy as np

from IPython.display import display, Markdown

from centre_config import SQL_USERS, SQL_SERVERS

import mosaiq_connection as msq_c

import mosaiq_field_export as msq_x
msq_x.use_mlc_missing_byte_workaround()


def display_fields(patient_ids):
    """Display all fields stored under a given Patient ID.
    """
    with msq_c.multi_mosaiq_connect(SQL_USERS, SQL_SERVERS) as cursors:
        for key in SQL_SERVERS:
            display(Markdown(
                '# {} Fields for {}'.format(key.upper(), patient_ids[key])))
            display(msq_x.patient_fields(cursors[key], patient_ids[key]))


def display_fields_overview(fields):
    """Display an overview for a given list of fields.
    """
    with msq_c.multi_mosaiq_connect(SQL_USERS, SQL_SERVERS) as cursors:
        for key in SQL_SERVERS:
            display(Markdown('# {} fields overview'.format(key.upper())))
            fields_overview = msq_x.create_fields_overview(
                cursors[key], fields[key])
            display(fields_overview)
            print("Combined MU: {}".format(np.sum(
                fields_overview['Total MU'].values.astype(float))))



def _pull_fields_sql_data(fields):
    """Pull the SQL data for all of the fields provided in the list.
    """
    with msq_c.multi_mosaiq_connect(SQL_USERS, SQL_SERVERS) as cursors:
        data = dict()
        for key, cursor in cursors.items():
            data[key] = msq_x.pull_sql_data(cursor, fields[key])

    return data


def _create_directory(directory):
    """Creates a directory if it doesn't already exist.
    """
    if not os.path.exists(directory):
        os.makedirs(directory)


def save_data_to_csv(directory, fields):
    """Pull the data for the given fields and save to a set of csv files.
    """
    _create_directory(directory)
    centre_data = _pull_fields_sql_data(fields)

    timestamp = '{:%Y%m%d_%H%M%S}'.format(datetime.datetime.now())
    timestamped_directory = os.path.join(directory, timestamp)
    _create_directory(timestamped_directory)

    for centre_key, data_container in centre_data.items():
        centre_directory = os.path.join(
            timestamped_directory, centre_key)
        _create_directory(centre_directory)

        testing_file_contents = [
            (item['filename'], item.pop('contents'))
            for item in data_container['testing']
        ]

        filepath = os.path.join(centre_directory, 'testing_metadata.json')
        with open(filepath, 'w') as file:
            file.write(json.dumps(
                data_container['testing'], indent=4, sort_keys=True))

        for file_name, file_contents in data_container['storage'].items():
            filepath = os.path.join(centre_directory, file_name)
            file_contents.to_csv(filepath)

        for file_name, file_contents in testing_file_contents:
            filepath = os.path.join(centre_directory, file_name)
            file_contents.to_csv(filepath)



from setuptools import setup

setup(
    name="mosaiq_transfer_check",
    version="0.2.0",
    author="Simon Biggs",
    author_email="mail@simonbiggs.net",
    description="Compare mosaiq entries for two different sites.",
    packages=[
        "mosaiq_transfer_check"
    ],
    entry_points={
        'console_scripts': [
            'mosaiq_transfer_check=mosaiq_transfer_check:main',
        ],
    },
    license='AGPL3+',
    install_requires=[
        'numpy',
        'pandas',
        'jinja2',
        'IPython',
        'csv_compare',
        'mosaiq_connection',
        'mosaiq_field_export',
        'notebook',
        'centre_config'
    ],
    include_package_data=True
)